<?php
/**
 * 用户 Token 及 refreshToken
 * @作者 苏相锟 <admin@phpok.com>
 * @主页 https://www.phpok.com
 * @版本 5.x
 * @授权 GNU Lesser General Public License https://www.phpok.com/lgpl.html 
 * @时间 2022年1月7日
**/

/**
 * 安全限制，防止直接访问
**/
if(!defined("PHPOK_SET")){
	exit("<h1>Access Denied</h1>");
}

class token_control extends phpok_control
{
	public function __construct()
	{
		parent::control();
	}

	public function index_f()
	{
		$this->refresh_f();
	}

	public function header_info($config)
	{
		$tmp = $_SERVER['HTTP_AUTHORIZATION'] ? $_SERVER['HTTP_AUTHORIZATION'] : $_SERVER['HTTP_TOKEN'];
		if(!$tmp){
			$this->error(P_Lang('HTTP_AUTHORIZATION 认证信息不存在'));
		}
		$tmps = explode(" ",$tmp);
		if(!$tmps[1]){
			$token = $tmp;
		}else{
			$token = $tmps[1];
		}
		list($type, $string) = explode(':', base64_decode($token));
		$this->lib('token')->etype('api_code');
		$this->lib('token')->keyid($config['api_code']);
		$this->lib('token')->expiry(24*60*60);
		$decode_data = $this->lib('token')->decode($string);
		if(!$decode_data || !is_array($decode_data)){
			$this->error(P_Lang('HTTP_AUTHORIZATION 认证失败'));
		}
		if(!$decode_data['session_id']){
			$this->error(P_Lang('HTTP_AUTHORIZATION 认证失败，Session 丢失'));
		}
		$this->session()->sessid($decode_data['session_id']);
		return true;
	}

	public function create_f()
	{
		$string = $this->get('data','html');
		if(!$string){
			$this->error(P_Lang('未接收到客户端发送的请求数据'));
		}
		$config = $this->model('config')->get_all();
		if(!$config || !$config['private_key'] || !$config['public_key'] || !$config['api_code']){
			$this->error(P_Lang('系统配置不全，请系统管理员到后台配置好公钥，密钥及API验证串'));
		}
		$this->lib("token")->etype("public_key");
		$this->lib('token')->public_key($config['public_key']);
		$this->lib('token')->private_key($config['private_key']);
		$this->lib('token')->expiry(365*24*60*60);
		$info = $this->lib('token')->decode($data);
		if(!$info){
			$this->error(P_Lang('密钥为空'));
		}
		$rs = is_string($info) ? $this->lib('json')->decode($info) : $info;
		$user = array();
		if($rs['user_id'] && is_numeric($rs['user_id']) && $rs['user_sign']){
			$user = $this->model('user')->get_one(intval($rs['user_id']))
			if($user && $user['id']){
				$tmpcode = md5($config['api_code'].'-'.$user['id'].'-'.$user['regtime']);
				if($tmpcode != $rs['user_sign']){
					$user = array();
				}
			}
			if($user && $user['id'] && ($user['status'] != 1 || $user['satus'] != 3)){
				$user = array();
			}
		}
		//强制退出
		if(isset($rs['user_id']) && !isset($user['id']) && $this->session->val('user_id')){
			$this->model('user')->logout();
		}
		//如果存在用户
		if(isset($user['id']) && $user['id']){
			$this->header_info($config);
			$this->model('user')->login($user);
		}
		$keyid = $rs['api_code'];
		//生成验签密码
		$data = array();
		$rand = $this->lib('common')->str_rand(16);
		$expire_time = 1200;
		if($this->config['token'] && $this->config['token']['time'] && $this->config['token']['time']>600){
			$expire_time = $this->config['token']['time'];
		}
		$expiry = $this->time + $expire_time;
		$data = array("time"=>$expiry,"value"=>$rand,'expire'=>$expire_time);
		if($user && $user['id']){
			$refresh_day = $this->config['refresh_day'] ? $this->config['refresh_day'] : 30;
			$refresh_id = 'refreshToken';
			if($this->config['token'] && $this->config['token']['refresh_id']){
				$refresh_id = $this->config['token']['refresh_id'];
			}
			$tmpdata = array();
			$tmpdata['time'] = $this->time + $refresh_day*24*60*60;
			$tmpdata['day'] = $refresh_day;
			$tmpdata['token'] = $this->refresh_token($config,$keyid,$user['id']);
			$tmpdata['user'] = $this->user_token($config,$user['id']);
			$data['refresh'] = $tmpdata;
		}
		//生成签名码，签名码不可解密，用于头部验证
		$data['token'] = $this->auth_token($config,$this->session->val('user_id'));	
		$this->lib("token")->etype("api_code");
		$this->lib('token')->keyid($keyid);
		$this->lib('token')->expiry($expire_time);
		$out = $this->lib('token')->encode($data);
		 //输出可解密的数据
		$this->success($out);
	}

	private function user_token($config,$user_id=0)
	{
		if(!$user_id || !$config){
			return false;
		}
		$user = $this->model('user')->get_one($user_id);
		if(!$user){
			return false;
		}
		unset($data['pass']);
		//生成新的密钥，防止窜改
		$user['_sign'] = md5($config['api_code'].'-'.$user['id'].'-'.$user['regtime']);
		$this->lib("token")->etype("public_key");
		$this->lib('token')->public_key($config['public_key']);
		$this->lib('token')->private_key($config['private_key']);
		$this->lib('token')->expiry(365*24*60*60);
		$info = $this->lib('token')->encode($user);
		return $info;
	}

	/**
	 * 生成签名码，签名码有效时间默认二个小时
	**/
	private function auth_token($config,$user_id=0)
	{
		$time = $this->time;
		$data = array();
		$data['session_id'] = $this->session->sessid();
		$data['time'] = $time;
		$data['ip'] = $this->lib('common')->ip();
		if($user_id){
			$data['user_id'] = $user_id;
		}
		ksort($data);
		$expire_time = 1200;
		if($this->config['token'] && $this->config['token']['time'] && $this->config['token']['time']>600){
			$expire_time = $this->config['token']['time'];
		}
		$this->lib('token')->etype('api_code');
		$this->lib('token')->keyid($config['api_code']);
		$this->lib('token')->expiry($expire_time);
		$info = $this->lib('token')->encode($data);
		$token = base64_encode('code:'.$info);
		return $token;
	}

	private function refresh_token($config,$keyid=0,$user_id=0)
	{
		$time = $this->time;
		$data = array();
		$data['keyid'] = $keyid;
		$data['time'] = $time;
		$data['user_id'] = $user_id;
		$data['sign'] = md5($config['api_code'].'-'.$time.'-',$user_id);
		$expire_time = 24*60*60;
		$refresh_day = ($this->config['refresh_day'] && $this->config['refresh_day']>10) ? $this->config['refresh_day'] : 30;
		$this->lib("token")->etype("api_code");
		$this->lib('token')->keyid($config['api_code']);
		$this->lib('token')->expiry($refresh_day*24*60*60);
		$info = $this->lib('token')->encode($data);
		return $info;
	}

	/**
	 * 通过 refresh 获取新的 Token
	**/
	public function refresh_f()
	{
		$token_id = ($this->config['token'] && $this->config['token']['refresh_id']) ? $this->config['token']['refresh_id'] : 'refreshToken';
		$day = ($this->config['token'] && $this->config['token']['refresh_day']) ? $this->config['token']['refresh_day'] : 30;
		$token = $this->get($token_id,'html');
		if(!$token){
			$this->error(P_Lang('未指定 refreshToken 信息'));
		}
		$config = $this->model('config')->get_all();
		if(!$config || !$config['private_key'] || !$config['public_key'] || !$config['api_code']){
			$this->error(P_Lang('系统配置不全，请系统管理员到后台配置好公钥，密钥及API验证串'));
		}
		$refresh_day = $this->config['refresh_day'] ? $this->config['refresh_day'] : 30;
		//对 token 数据进行解密
		$this->lib("token")->etype("api_code");
		$this->lib('token')->keyid($config['api_code']);
		$this->lib('token')->expiry($refresh_day*24*60*60);
		$decode_tmp = $this->lib('token')->decode($token);
		if(!$decode_tmp){
			if($this->session->val('user_id')){
				$this->model('user')->logout();
			}
			$this->error(P_Lang('解密失败，请检查'));
		}
		$decode_data = is_string($decode_tmp) ? $this->lib('json')->decode($decode_tmp) : $decode_tmp;
		if(!$decode_data['user_id'] || $decode_data['keyid']){
			if($this->session->val('user_id')){
				$this->model('user')->logout();
			}
			$this->error(P_Lang('解密失败，请检查'));
		}
		$user = $this->model('user')->get_one($decode_data['user_id']);
		if(!$user || !$user['id'] || ($user['status']!=1 && $user['status'] != 3)){
			if($this->session->val('user_id')){
				$this->model('user')->logout();
			}
			$this->error(P_Lang('用户不存在或未审核或被禁用'));
		}
		//更新头部，解决自定义 session
		$this->header_info($config);
		$this->model('user')->login($user);
		$rand = $this->lib('common')->str_rand(16);
		//生成新的加密
		$expire_time = 1200;
		if($this->config['token'] && $this->config['token']['time'] && $this->config['token']['time']>600){
			$expire_time = $this->config['token']['time'];
		}
		$expiry = $this->time + $expire_time;
		$data = array("time"=>$expiry,"value"=>$rand,'expire'=>$expire_time);
		if($decode_data && $decode_data['user_id']){
			$tmpdata = array();
			$tmpdata['time'] = $this->time + $refresh_day*24*60*60;
			$tmpdata['day'] = $refresh_day;
			$tmpdata['token'] = $this->refresh_token($config,$decode_data['keyid'],$decode_data['user_id']);
			$tmpdata['user'] = $this->user_token($config,$decode_data['user_id']);
			$data['refresh'] = $tmpdata;
		}
		//生成签名码，签名码不可解密，用于头部验证
		$data['token'] = $this->auth_token($config,$decode_data['user_id']);	
		$this->lib("token")->etype("api_code");
		$this->lib('token')->keyid($decode_data['keyid']);
		$this->lib('token')->expiry($expire_time);
		$out = $this->lib('token')->encode($data);
		$this->success($out);
	}

	public function demo_f()
	{
		$config = $this->model('config')->get_all();
		if(!$config || !$config['private_key'] || !$config['public_key']){
			$this->error(P_Lang('系统配置不全，请系统管理员到后台配置好公钥，密钥及API验证串'));
		}
		$data = array();
		$data['title'] = '测试加密解密';
		$this->lib("token")->etype("public_key");
		$this->lib('token')->public_key($config['public_key']);
		$this->lib('token')->private_key($config['private_key']);
		$this->lib('token')->expiry(24*60*60);
		$tmp = $this->lib('token')->encode($data);
		$this->lib("token")->etype("public_key");
		$list = $this->lib('token')->decode($tmp);
		$this->success(array('encode'=>$tmp,'decode'=>$list));
	}
}
